@@ -12,55 +12,45 @@ class DemoParser:
 
     Attributes:
         demofile (string): A string denoting the path to the demo file, which ends in .dem
-        log (boolean): A boolean denoting if a log will be written. If true, log is written to "csgo_parser.log"
         demo_id (string): A unique demo name/game id. Default is inferred from demofile name
+        log (boolean): A boolean indicating if the log should print to stdout.
         parse_rate (int): One of 128, 64, 32, 16, 8, 4, 2, or 1. The lower the value, the more frames are collected. Indicates spacing between parsed demo frames in ticks. Default is 128.
         parse_frames (bool): Flag if you want to parse frames (trajectory data) or not
+        parse_kill_frames (bool): Flag if you want to parse frames on kills
         trade_time (int): Length of the window for a trade (in seconds). Default is 5.
         dmg_rolled (bool): Boolean if you want damages rolled up (since multiple damages for a player can happen in 1 tick from the same weapon.)
         buy_style (string): Buy style string, one of "hltv" or "csgo"
 
     Raises:
-        ValueError: Raises a ValueError if the Golang version is lower than 1.14
+        ValueError: Raises a ValueError if the Golang version is lower than 1.17
     """
 
     def __init__(
         self,
         demofile="",
         outpath=None,
-        log=False,
         demo_id=None,
+        log=False,
         parse_rate=128,
         parse_frames=True,
+        parse_kill_frames=False,
         trade_time=5,
         dmg_rolled=False,
         buy_style="hltv",
         json_indentation=False,
     ):
         # Set up logger
-        if log:
-            logging.basicConfig(
-                filename="csgo_demoparser.log",
-                level=logging.INFO,
-                format="%(asctime)s [%(levelname)s] %(message)s",
-                datefmt="%H:%M:%S",
-            )
-            self.logger = logging.getLogger("CSGODemoParser")
-            self.logger.handlers = []
-            fh = logging.FileHandler("csgo_demoparser.log")
-            fh.setLevel(logging.INFO)
-            self.logger.addHandler(fh)
-        else:
-            logging.basicConfig(
-                level=logging.INFO,
-                format="%(asctime)s [%(levelname)s] %(message)s",
-                datefmt="%H:%M:%S",
-            )
-            self.logger = logging.getLogger("CSGODemoParser")
+        logging.basicConfig(
+            level=logging.INFO,
+            format="%(asctime)s [%(levelname)s] %(message)s",
+            datefmt="%H:%M:%S",
+        )
+        self.logger = logging.getLogger("awpy")
+        self.logger.propagate = log
 
         # Handle demofile and demo_id name. Finds right most '/' in case demofile is a specified path.
         self.demofile = os.path.abspath(demofile)
-        self.logger.info("Initialized CSGODemoParser with demofile " + self.demofile)
+        self.logger.info("Initialized awpy DemoParser with demofile " + self.demofile)
         if (demo_id is None) | (demo_id == ""):
             self.demo_id = demofile[demofile.rfind("/") + 1 : -4]
         else:
@@ -123,9 +113,11 @@ class DemoParser:
 
         self.dmg_rolled = dmg_rolled
         self.parse_frames = parse_frames
+        self.parse_kill_frames = parse_kill_frames
         self.json_indentation = json_indentation
         self.logger.info("Rollup damages set to " + str(self.dmg_rolled))
         self.logger.info("Parse frames set to " + str(self.parse_frames))
+        self.logger.info("Parse kill frames set to " + str(self.parse_kill_frames))
         self.logger.info("Output json indentation set to " + str(self.json_indentation))
 
         # Set parse error to False
@@ -138,20 +130,20 @@ class DemoParser:
             Outputs a JSON file to current working directory.
 
         Raises:
-            ValueError: Raises a ValueError if the Golang version is lower than 1.14
+            ValueError: Raises a ValueError if the Golang version is lower than 1.17
             FileNotFoundError: Raises a FileNotFoundError if the demofile path does not exist.
         """
         # Check if Golang version is compatible
         acceptable_go = check_go_version()
         if not acceptable_go:
             self.logger.error(
-                "Error calling Go. Check if Go is installed using 'go version'. Need at least v1.14.0."
+                "Error calling Go. Check if Go is installed using 'go version'. Need at least v1.17.0."
             )
             raise ValueError(
-                "Error calling Go. Check if Go is installed using 'go version'. Need at least v1.14.0."
+                "Error calling Go. Check if Go is installed using 'go version'. Need at least v1.17.0."
             )
         else:
-            self.logger.info("Go version>=1.14.0")
+            self.logger.info("Go version>=1.17.0")
 
         # Check if demofile exists
         if not os.path.exists(os.path.abspath(self.demofile)):
@@ -182,6 +174,8 @@ class DemoParser:
             self.parser_cmd.append("--dmgrolled")
         if self.parse_frames:
             self.parser_cmd.append("--parseframes")
+        if self.parse_kill_frames:
+            self.parser_cmd.append("--parsekillframes")
         if self.json_indentation:
             self.parser_cmd.append("--jsonindentation")
         proc = subprocess.Popen(
@@ -191,7 +185,7 @@ class DemoParser:
         )
         stdout = proc.stdout.read().splitlines()
         self.output_file = self.demo_id + ".json"
-        if os.path.isfile(self.output_file):
+        if os.path.isfile(self.outpath + "/" + self.output_file):
             self.logger.info("Wrote demo parse output to " + self.output_file)
             self.parse_error = False
         else:
@@ -212,7 +206,7 @@ class DemoParser:
             FileNotFoundError: Raises a FileNotFoundError if the JSON path doesn't exist
         """
         # Check if JSON exists
-        if not os.path.exists(os.path.abspath(json_path)):
+        if not os.path.exists(json_path):
             self.logger.error("JSON path does not exist!")
             raise FileNotFoundError("JSON path does not exist!")
 
@@ -226,11 +220,12 @@ class DemoParser:
         )
         return demo_data
 
-    def parse(self, return_type="json"):
+    def parse(self, return_type="json", clean=True):
         """Wrapper for parse_demo() and read_json(). Use to parse a demo.
 
         Args:
             return_type (string): Either "json" or "df"
+            clean (bool, optional): True to run clean_rounds, otherwise, uncleaned data is returned. Defaults to True.
 
         Returns:
             A dictionary of output (which is parsed to a JSON file in the working directory)
@@ -241,6 +236,8 @@ class DemoParser:
         """
         self.parse_demo()
         self.read_json(json_path=self.outpath + "/" + self.output_file)
+        if clean:
+            clean_data = self.clean_rounds()
         if self.json:
             self.logger.info("JSON output found")
             if return_type == "json":
@@ -272,15 +269,60 @@ class DemoParser:
             demo_data["mapName"] = self.json["mapName"]
             demo_data["tickRate"] = self.json["tickRate"]
             demo_data["playbackTicks"] = self.json["playbackTicks"]
+            # Rounds
             demo_data["rounds"] = self._parse_rounds()
+            # Kills
             demo_data["kills"] = self._parse_kills()
+            demo_data["kills"]["attackerSteamID"] = demo_data["kills"][
+                "attackerSteamID"
+            ].astype(pd.Int64Dtype())
+            demo_data["kills"]["victimSteamID"] = demo_data["kills"][
+                "victimSteamID"
+            ].astype(pd.Int64Dtype())
+            demo_data["kills"]["assisterSteamID"] = demo_data["kills"][
+                "assisterSteamID"
+            ].astype(pd.Int64Dtype())
+            demo_data["kills"]["flashThrowerSteamID"] = demo_data["kills"][
+                "flashThrowerSteamID"
+            ].astype(pd.Int64Dtype())
+            # Damages
             demo_data["damages"] = self._parse_damages()
+            demo_data["damages"]["attackerSteamID"] = demo_data["damages"][
+                "attackerSteamID"
+            ].astype(pd.Int64Dtype())
+            demo_data["damages"]["victimSteamID"] = demo_data["damages"][
+                "victimSteamID"
+            ].astype(pd.Int64Dtype())
+            # Grenades
             demo_data["grenades"] = self._parse_grenades()
+            demo_data["grenades"]["throwerSteamID"] = demo_data["grenades"][
+                "throwerSteamID"
+            ].astype(pd.Int64Dtype())
+            # Flashes
             demo_data["flashes"] = self._parse_flashes()
+            demo_data["flashes"]["attackerSteamID"] = demo_data["flashes"][
+                "attackerSteamID"
+            ].astype(pd.Int64Dtype())
+            demo_data["flashes"]["playerSteamID"] = demo_data["flashes"][
+                "playerSteamID"
+            ].astype(pd.Int64Dtype())
+            # Weapon Fires
             demo_data["weaponFires"] = self._parse_weapon_fires()
+            demo_data["weaponFires"]["playerSteamID"] = demo_data["flashes"][
+                "playerSteamID"
+            ].astype(pd.Int64Dtype())
+            # Bomb Events
             demo_data["bombEvents"] = self._parse_bomb_events()
+            demo_data["bombEvents"]["playerSteamID"] = demo_data["bombEvents"][
+                "playerSteamID"
+            ].astype(pd.Int64Dtype())
+            # Frames
             demo_data["frames"] = self._parse_frames()
+            # Player Frames
             demo_data["playerFrames"] = self._parse_player_frames()
+            demo_data["playerFrames"]["steamID"] = demo_data["playerFrames"][
+                "steamID"
+            ].astype(pd.Int64Dtype())
             self.logger.info("Returned dataframe output")
             return demo_data
         else:
@@ -405,16 +447,14 @@ class DemoParser:
                 "winningTeam",
                 "losingTeam",
                 "roundEndReason",
-                "tStartEqVal",
-                "tRoundStartEqVal",
-                "tRoundStartMoney",
-                "tBuyType",
-                "tSpend",
-                "ctStartEqVal",
+                "ctFreezeTimeEndEqVal",
                 "ctRoundStartEqVal",
-                "ctRoundStartMoney",
+                "ctRoundSpendMoney",
                 "ctBuyType",
-                "ctSpend",
+                "tFreezeTimeEndEqVal",
+                "tRoundStartEqVal",
+                "tRoundSpendMoney",
+                "tBuyType",
             ]
             for r in self.json["gameRounds"]:
                 round_item = {}
@@ -609,6 +649,7 @@ class DemoParser:
         remove_excess_players=True,
         remove_excess_kills=True,
         remove_bad_endings=True,
+        remove_bad_scoring=True,
         return_type="json",
         save_to_json=True,
     ):
@@ -622,6 +663,7 @@ class DemoParser:
             remove_excess_players (bool, optional): Remove rounds with more than 5 players. Defaults to True.
             remove_excess_kills (bool, optional): Remove rounds with more than 10 kills. Defaults to True.
             remove_bad_endings (bool, optional): Remove rounds with bad round end reasons. Defaults to True.
+            remove_bad_scoring (bool, optional): Remove rounds where the scoring is off (like scores going below the previous round's). Defaults to False.
             return_type (str, optional): Return JSON or DataFrame. Defaults to "json".
             save_to_json (bool, optional): Whether to write the JSON to a file. Defaults to True.
 
@@ -646,6 +688,8 @@ class DemoParser:
                 self.remove_excess_kill_rounds()
             if remove_bad_endings:
                 self.remove_end_round()
+            if remove_bad_scoring:
+                self.remove_bad_scoring()
             self.renumber_rounds()
             # self.rescore_rounds() -- Need to edit to take into account half switches
             if save_to_json:
@@ -666,7 +710,7 @@ class DemoParser:
 
     def write_json(self):
         """Rewrite the JSON file"""
-        with open(self.output_file, "w", encoding="utf8") as fp:
+        with open(self.outpath + "/" + self.output_file, "w", encoding="utf8") as fp:
             json.dump(self.json, fp, indent=(1 if self.json_indentation else None))
 
     def renumber_rounds(self):
@@ -732,6 +776,59 @@ class DemoParser:
                 "JSON not found. Run .parse() or .read_json() if JSON already exists"
             )
 
+    def remove_bad_scoring(self):
+        """Removes rounds where the scoring is bad.
+
+        We loop through the rounds:
+        If the round ahead has equal or less score, we do not add the current round.
+        If the round ahead has +1 score, we add the current round
+
+        Raises:
+            AttributeError: Raises an AttributeError if the .json attribute is None
+        """
+        if self.json:
+            cleaned_rounds = []
+            for i, r in enumerate(self.json["gameRounds"]):
+                current_round_total = (
+                    r["tScore"] + r["endTScore"] + r["ctScore"] + r["endCTScore"]
+                )
+                if i < len(self.json["gameRounds"]) - 1:
+                    lookahead_round = self.json["gameRounds"][i + 1]
+                    lookahead_round_total = (
+                        lookahead_round["tScore"]
+                        + lookahead_round["endTScore"]
+                        + lookahead_round["ctScore"]
+                        + lookahead_round["endCTScore"]
+                    )
+                    if lookahead_round_total > current_round_total:
+                        cleaned_rounds.append(r)
+                    elif (r["endTScore"] == 16) & (r["endCTScore"] <= 14):
+                        cleaned_rounds.append(r)
+                    elif (r["endCTScore"] == 16) & (r["endTScore"] <= 14):
+                        cleaned_rounds.append(r)
+                else:
+                    lookback_round = self.json["gameRounds"][i]
+                    lookback_round_total = (
+                        lookback_round["tScore"]
+                        + lookback_round["endTScore"]
+                        + lookback_round["ctScore"]
+                        + lookback_round["endCTScore"]
+                    )
+                    if (r["endTScore"] == 16) & (r["endCTScore"] <= 14):
+                        cleaned_rounds.append(r)
+                    elif (r["endCTScore"] == 16) & (r["endTScore"] <= 14):
+                        cleaned_rounds.append(r)
+                    elif current_round_total > lookback_round_total:
+                        cleaned_rounds.append(r)
+            self.json["gameRounds"] = cleaned_rounds
+        else:
+            self.logger.error(
+                "JSON not found. Run .parse() or .read_json() if JSON already exists"
+            )
+            raise AttributeError(
+                "JSON not found. Run .parse() or .read_json() if JSON already exists"
+            )
+
     def remove_rounds_with_no_frames(self):
         """Removes rounds with no frames
 
@@ -857,13 +954,14 @@ class DemoParser:
         if self.json:
             cleaned_rounds = []
             for r in self.json["gameRounds"]:
-                if not r["isWarmup"] and r["kills"]:
+                if not r["isWarmup"]:
                     total_kills = len(r["kills"])
                     total_knife_kills = 0
-                    for k in r["kills"]:
-                        if k["weapon"] == "Knife":
-                            total_knife_kills += 1
-                    if total_knife_kills != total_kills:
+                    if total_kills > 0:
+                        for k in r["kills"]:
+                            if k["weapon"] == "Knife":
+                                total_knife_kills += 1
+                    if (total_knife_kills != total_kills) | (total_knife_kills == 0):
                         cleaned_rounds.append(r)
             self.json["gameRounds"] = cleaned_rounds
         else:
@@ -883,11 +981,9 @@ class DemoParser:
         if self.json:
             cleaned_rounds = []
             for r in self.json["gameRounds"]:
-                if not r["isWarmup"] and r["kills"]:
-                    if len(r["kills"]) > 0:
-                        total_kills = len(r["kills"])
-                        if total_kills <= 10:
-                            cleaned_rounds.append(r)
+                if not r["isWarmup"]:
+                    if len(r["kills"]) <= 10:
+                        cleaned_rounds.append(r)
             self.json["gameRounds"] = cleaned_rounds
         else:
             self.logger.error(
