@@ -34,11 +34,12 @@ type Game struct {
 
 // ParserOpts holds the parameters passed to the parser
 type ParserOpts struct {
-	ParseRate     int          `json:"parseRate"`
-	ParseFrames   bool         `json:"parseFrames"`
-	TradeTime     int64        `json:"tradeTime"`
-	RoundBuyStyle string       `json:"roundBuyStyle"`
-	DamagesRolled bool         `json:"damagesRolledUp"`
+	ParseRate       int    `json:"parseRate"`
+	ParseFrames     bool   `json:"parseFrames"`
+	ParseKillFrames bool   `json:"parseKillFrames"`
+	TradeTime       int64  `json:"tradeTime"`
+	RoundBuyStyle   string `json:"roundBuyStyle"`
+	DamagesRolled   bool   `json:"damagesRolledUp"`
 }
 
 // MatchPhases holds lists of when match events occurred
@@ -97,40 +98,52 @@ type ConnectAction struct {
 
 // GameRound contains round info and events
 type GameRound struct {
-	RoundNum          int64              `json:"roundNum"`
-	IsWarmup          bool               `json:"isWarmup"`
-	StartTick         int64              `json:"startTick"`
-	FreezeTimeEndTick int64              `json:"freezeTimeEndTick"`
-	EndTick           int64              `json:"endTick"`
-	EndOfficialTick   int64              `json:"endOfficialTick"`
-	BombPlantTick     *int64             `json:"bombPlantTick"`
-	TScore            int64              `json:"tScore"`
-	CTScore           int64              `json:"ctScore"`
-	EndTScore         int64              `json:"endTScore"`
-	EndCTScore        int64              `json:"endCTScore"`
-	CTTeam            *string            `json:"ctTeam"`
-	TTeam             *string            `json:"tTeam"`
-	WinningSide       string             `json:"winningSide"`
-	WinningTeam       *string            `json:"winningTeam"`
-	LosingTeam        *string            `json:"losingTeam"`
-	Reason            string             `json:"roundEndReason"`
-	CTStartEqVal      int64              `json:"ctStartEqVal"`
-	CTBeginEqVal      int64              `json:"ctRoundStartEqVal"`
-	CTBeginMoney      int64              `json:"ctRoundStartMoney"`
-	CTBuyType         string             `json:"ctBuyType"`
-	CTSpend           int64              `json:"ctSpend"`
-	TStartEqVal       int64              `json:"tStartEqVal"`
-	TBeginEqVal       int64              `json:"tRoundStartEqVal"`
-	TBeginMoney       int64              `json:"tRoundStartMoney"`
-	TBuyType          string             `json:"tBuyType"`
-	TSpend            int64              `json:"tSpend"`
-	Kills             []KillAction       `json:"kills"`
-	Damages           []DamageAction     `json:"damages"`
-	Grenades          []GrenadeAction    `json:"grenades"`
-	Bomb              []BombAction       `json:"bombEvents"`
-	WeaponFires       []WeaponFireAction `json:"weaponFires"`
-	Flashes           []FlashAction      `json:"flashes"`
-	Frames            []GameFrame        `json:"frames"`
+	RoundNum             int64              `json:"roundNum"`
+	IsWarmup             bool               `json:"isWarmup"`
+	StartTick            int64              `json:"startTick"`
+	FreezeTimeEndTick    int64              `json:"freezeTimeEndTick"`
+	EndTick              int64              `json:"endTick"`
+	EndOfficialTick      int64              `json:"endOfficialTick"`
+	BombPlantTick        *int64             `json:"bombPlantTick"`
+	TScore               int64              `json:"tScore"`
+	CTScore              int64              `json:"ctScore"`
+	EndTScore            int64              `json:"endTScore"`
+	EndCTScore           int64              `json:"endCTScore"`
+	CTTeam               *string            `json:"ctTeam"`
+	TTeam                *string            `json:"tTeam"`
+	WinningSide          string             `json:"winningSide"`
+	WinningTeam          *string            `json:"winningTeam"`
+	LosingTeam           *string            `json:"losingTeam"`
+	Reason               string             `json:"roundEndReason"`
+	CTFreezeTimeEndEqVal int64              `json:"ctFreezeTimeEndEqVal"`
+	CTRoundStartEqVal    int64              `json:"ctRoundStartEqVal"`
+	CTRoundMoneySpend    int64              `json:"ctRoundSpendMoney"`
+	CTBuyType            string             `json:"ctBuyType"`
+	TFreezeTimeEndEqVal  int64              `json:"tFreezeTimeEndEqVal"`
+	TRoundStartEqVal     int64              `json:"tRoundStartEqVal"`
+	TRoundMoneySpend     int64              `json:"tRoundSpendMoney"`
+	TBuyType             string             `json:"tBuyType"`
+	CTSide               PlayerTeams        `json:"ctSide"`
+	TSide                PlayerTeams        `json:"tSide"`
+	Kills                []KillAction       `json:"kills"`
+	Damages              []DamageAction     `json:"damages"`
+	Grenades             []GrenadeAction    `json:"grenades"`
+	Bomb                 []BombAction       `json:"bombEvents"`
+	WeaponFires          []WeaponFireAction `json:"weaponFires"`
+	Flashes              []FlashAction      `json:"flashes"`
+	Frames               []GameFrame        `json:"frames"`
+}
+
+// PlayerTeam
+type PlayerTeams struct {
+	TeamName string     `json:"teamName"`
+	Players  []Players  `json:"players"`
+}
+
+// Players
+type Players struct {
+	PlayerName string `json:"playerName"`
+	SteamID    int64  `json:"steamID"`
 }
 
 // GrenadeAction events
@@ -195,6 +208,7 @@ type DamageAction struct {
 	VictimViewX      *float64 `json:"victimViewX"`
 	VictimViewY      *float64 `json:"victimViewY"`
 	Weapon           string   `json:"weapon"`
+	WeaponClass      string   `json:"weaponClass"`
 	HpDamage         int64    `json:"hpDamage"`
 	HpDamageTaken    int64    `json:"hpDamageTaken"`
 	ArmorDamage      int64    `json:"armorDamage"`
@@ -252,6 +266,7 @@ type KillAction struct {
 	PlayerTradedTeam    *string  `json:"playerTradedTeam"`
 	PlayerTradedSteamID *int64   `json:"playerTradedSteamID"`
 	Weapon              string   `json:"weapon"`
+	WeaponClass         string   `json:"weaponClass"`
 }
 
 // WeaponFireAction events
@@ -270,6 +285,9 @@ type WeaponFireAction struct {
 	PlayerViewY    float64 `json:"playerViewY"`
 	PlayerStrafe   bool    `json:"playerStrafe"`
 	Weapon         string  `json:"weapon"`
+	WeaponClass    string  `json:"weaponClass"`
+	AmmoInMagazine int64   `json:"ammoInMagazine"`
+	AmmoInReserve  int64   `json:"ammoInReserve"`
 	ZoomLevel      int64   `json:"zoomLevel"`
 }
 
@@ -301,22 +319,41 @@ type FlashAction struct {
 
 // GameFrame (game state at time t)
 type GameFrame struct {
+	IsKillFrame bool          `json:"isKillFrame"`
 	Tick        int64         `json:"tick"`
 	Second      float64       `json:"seconds"`
 	ClockTime   string        `json:"clockTime"`
 	T           TeamFrameInfo `json:"t"`
 	CT          TeamFrameInfo `json:"ct"`
-	World       []WorldObject `json:"world"`
 	BombPlanted bool          `json:"bombPlanted"`
 	BombSite    string        `json:"bombsite"`
+	Bomb        BombInfo      `json:"bomb"`
+	Projectiles []GrenadeInfo `json:"projectiles"`
+	Smokes      []Smoke       `json:"smokes"`
+	Fires       []Fire        `json:"fires"`
 }
 
-// WorldObject in the world, like a bomb
-type WorldObject struct {
-	ObjType  string  `json:"objectType"`
-	X        float64 `json:"x"`
-	Y        float64 `json:"y"`
-	Z        float64 `json:"z"`
+// Bomb location
+type BombInfo struct {
+	X float64 `json:"x"`
+	Y float64 `json:"y"`
+	Z float64 `json:"z"`
+}
+
+// Projectile
+type GrenadeInfo struct {
+	ProjectileType  string  `json:"projectileType"`
+	X               float64 `json:"x"`
+	Y               float64 `json:"y"`
+	Z               float64 `json:"z"`
+}
+
+// Inferno from molly or incend. grenade
+type Fire struct {
+	UniqueID      int64   `json:"uniqueID"`
+	X             float64 `json:"x"`
+	Y             float64 `json:"y"`
+	Z             float64 `json:"z"`
 }
 
 // TeamFrameInfo at time t
@@ -385,6 +422,15 @@ type WeaponInfo struct {
 	AmmoInReserve  int64  `json:"ammoInReserve"`
 }
 
+// Smoke holds current smoke info
+type Smoke struct {
+	GrenadeEntityID int64   `json:"grenadeEntityID"`
+	StartTick       int64   `json:"startTick"`
+	X               float64 `json:"x"`
+	Y               float64 `json:"y"`
+	Z               float64 `json:"z"`
+}
+
 func convertRank(r int) string {
 	switch rank := r; rank {
 	case -1:
@@ -799,6 +845,17 @@ func countUtility(players []PlayerInfo) int64 {
 	return totalUtility
 }
 
+func sumPlayerEqVal(players []PlayerInfo) int64 {
+	var totalEqVal int64
+	totalEqVal = 0
+	for _, p := range players {
+		if p.IsAlive {
+			totalEqVal = totalEqVal + p.EqVal
+		}
+	}
+	return totalEqVal
+}
+
 func findIdx(sl []string, val string) int {
 	for p, v := range sl {
 		if v == val {
@@ -826,6 +883,11 @@ func stringInSlice(a string, list []string) bool {
     return false
 }
 
+func removeExpiredSmoke(s []Smoke, i int) []Smoke {
+    s[i] = s[len(s)-1]
+    return s[:len(s)-1]
+}
+
 // Main
 func main() {
 	/* Parse the arguments
@@ -839,6 +901,7 @@ func main() {
 	demoPathPtr := fl.String("demo", "", "Demo file `path`")
 	parseRatePtr := fl.Int("parserate", 128, "Parse rate, indicates spacing between ticks")
 	parseFramesPtr := fl.Bool("parseframes", false, "Parse frames")
+	parseKillFramesPtr := fl.Bool("parsekillframes", false, "Parse kill frames")
 	tradeTimePtr := fl.Int("tradetime", 5, "Trade time frame (in seconds)")
 	roundBuyPtr := fl.String("buystyle", "hltv", "Round buy style")
 	damagesRolledPtr := fl.Bool("dmgrolled", false, "Roll up damages")
@@ -852,6 +915,7 @@ func main() {
 	demPath := *demoPathPtr
 	parseRate := *parseRatePtr
 	parseFrames := *parseFramesPtr
+	parseKillFrames := *parseKillFramesPtr
 	tradeTime := int64(*tradeTimePtr)
 	roundBuyStyle := *roundBuyPtr
 	damagesRolled := *damagesRolledPtr
@@ -875,7 +939,7 @@ func main() {
 	currentMap := header.MapName
 	currentMap = cleanMapName(currentMap)
 
-	// Create flags to guIDe parsing
+	// Create flags to guide parsing
 	roundStarted := 0
 	roundInEndTime := 0
 	roundInFreezetime := 0
@@ -895,6 +959,9 @@ func main() {
 	currentGame.PlaybackFrames = int64(header.PlaybackFrames)
 	currentGame.ClientName = header.ClientName
 
+	// Create empty smoke tracking list
+	smokes := []Smoke{}
+
 	// Set parsing options
 	parsingOpts := ParserOpts{}
 	parsingOpts.ParseRate = int(parseRate)
@@ -1021,6 +1088,36 @@ func main() {
 		currentGame.MatchPhases.TeamSwitch = append(currentGame.MatchPhases.TeamSwitch, int64(gs.IngameTick()))
 	})
 
+	// Parse smokes
+	p.RegisterEventHandler(func(e events.SmokeStart) {
+		gs := p.GameState()
+		s := Smoke{}
+		s.GrenadeEntityID = e.Grenade.UniqueID() // GrenadeEntityID
+		s.StartTick = int64(gs.IngameTick())
+		s.X = float64(e.Position.X)
+		s.Y = float64(e.Position.Y)
+		s.Z = float64(e.Position.Z)
+		foundNade := false
+		for _, ele := range smokes {
+			if ele.GrenadeEntityID == s.GrenadeEntityID {
+				foundNade = true
+			}
+		}
+		if !foundNade {
+			smokes = append(smokes, s)
+		}
+		foundNade = false
+	})
+
+	p.RegisterEventHandler(func(e events.SmokeExpired) {
+		removeID := e.Grenade.UniqueID() // e.GrenadeEntityID
+		for i, ele := range smokes {
+			if ele.GrenadeEntityID == removeID {
+				smokes = removeExpiredSmoke(smokes, i)
+			}
+		}
+	})
+
 	// Parse round starts
 	p.RegisterEventHandler(func(e events.RoundStart) {
 		gs := p.GameState()
@@ -1035,6 +1132,9 @@ func main() {
 		roundInEndTime = 0
 		currentRound = GameRound{}
 
+		// Reset smokes
+		smokes = []Smoke{}
+
 		// Create empty action lists
 		currentRound.Bomb = []BombAction{}
 		currentRound.Damages = []DamageAction{}
@@ -1058,21 +1158,58 @@ func main() {
 		} 
 		
 		// Parse round money
-		tPlayers := gs.TeamTerrorists().Members()
-		currentRound.TBeginMoney = 0
-		ctPlayers := gs.TeamCounterTerrorists().Members()
-		currentRound.CTBeginMoney = 0
-		for _, p := range tPlayers {
-			if p != nil {
-				currentRound.TBeginMoney += int64(p.Money())
-			}
+		// tPlayers := gs.TeamTerrorists().Members()
+		// currentRound.TBeginMoney = 0
+		// ctPlayers := gs.TeamCounterTerrorists().Members()
+		// currentRound.CTBeginMoney = 0
+		// for _, p := range tPlayers {
+		// 	if p != nil {
+		// 		currentRound.TBeginMoney += int64(p.Money())
+		// 	}
 
-		}
-		for _, p := range ctPlayers {
-			if p != nil {
-				currentRound.CTBeginMoney += int64(p.Money())
+		// }
+		// for _, p := range ctPlayers {
+		// 	if p != nil {
+		// 		currentRound.CTBeginMoney += int64(p.Money())
+		// 	}
+		// }
+
+		// Parse the players
+		teamCT := PlayerTeams{}
+		teamCT.TeamName = gs.TeamCounterTerrorists().ClanName()
+		for _, player := range gs.TeamCounterTerrorists().Members() {
+			pl := Players{}
+			pl.PlayerName = player.Name
+			pl.SteamID = int64(player.SteamID64)
+			foundPlayer := false
+			for _, p := range teamCT.Players {
+				if p.SteamID == pl.SteamID {
+					foundPlayer = true
+				}
+			}
+			if !foundPlayer {
+				teamCT.Players = append(teamCT.Players, pl)
 			}
 		}
+		currentRound.CTSide = teamCT
+
+		teamT := PlayerTeams{}
+		teamT.TeamName = gs.TeamTerrorists().ClanName()
+		for _, player := range gs.TeamTerrorists().Members() {
+			pl := Players{}
+			pl.PlayerName = player.Name
+			pl.SteamID = int64(player.SteamID64)
+			foundPlayer := false
+			for _, p := range teamT.Players {
+				if p.SteamID == pl.SteamID {
+					foundPlayer = true
+				}
+			}
+			if !foundPlayer {
+				teamT.Players = append(teamT.Players, pl)
+			}
+		}
+		currentRound.TSide = teamT
 	})
 
 	// Parse round freezetime ends
@@ -1090,6 +1227,9 @@ func main() {
 			currentRound.CTTeam = &ctTeam
 		}
 
+		// Reset smokes
+		smokes = []Smoke{}
+
 		// Determine if round is still in warmup mode
 		currentRound.IsWarmup = gs.IsWarmupPeriod()
 
@@ -1163,24 +1303,44 @@ func main() {
 					currentRound.CTScore = int64(gs.TeamCounterTerrorists().Score())
 				}
 			}
+		}
 
-			// Parse round money
-			tPlayers := gs.TeamTerrorists().Members()
-			currentRound.TBeginMoney = 0
-			ctPlayers := gs.TeamCounterTerrorists().Members()
-			currentRound.CTBeginMoney = 0
-			for _, p := range tPlayers {
-				if p != nil {
-					currentRound.TBeginMoney += int64(p.Money())
+		// Parse the players
+		teamCT := PlayerTeams{}
+		teamCT.TeamName = gs.TeamCounterTerrorists().ClanName()
+		for _, player := range gs.TeamCounterTerrorists().Members() {
+			pl := Players{}
+			pl.PlayerName = player.Name
+			pl.SteamID = int64(player.SteamID64)
+			foundPlayer := false
+			for _, p := range teamCT.Players {
+				if p.SteamID == pl.SteamID {
+					foundPlayer = true
 				}
-
 			}
-			for _, p := range ctPlayers {
-				if p != nil {
-					currentRound.CTBeginMoney += int64(p.Money())
-				}
+			if !foundPlayer {
+				teamCT.Players = append(teamCT.Players, pl)
 			}
 		}
+		currentRound.CTSide = teamCT
+
+		teamT := PlayerTeams{}
+		teamT.TeamName = gs.TeamTerrorists().ClanName()
+		for _, player := range gs.TeamTerrorists().Members() {
+			pl := Players{}
+			pl.PlayerName = player.Name
+			pl.SteamID = int64(player.SteamID64)
+			foundPlayer := false
+			for _, p := range teamT.Players {
+				if p.SteamID == pl.SteamID {
+					foundPlayer = true
+				}
+			}
+			if !foundPlayer {
+				teamT.Players = append(teamT.Players, pl)
+			}
+		}
+		currentRound.TSide = teamT
 
 		roundInFreezetime = 0
 		currentRound.FreezeTimeEndTick = int64(gs.IngameTick())
@@ -1193,19 +1353,13 @@ func main() {
 		if roundInEndTime == 0 {
 			currentRound.EndTick = int64(gs.IngameTick()) - (RoundRestartDelay * currentGame.TickRate)
 			currentRound.EndOfficialTick = int64(gs.IngameTick())
-			currentRound.CTBeginEqVal = int64(gs.TeamCounterTerrorists().RoundStartEquipmentValue())
-			currentRound.TBeginEqVal = int64(gs.TeamTerrorists().RoundStartEquipmentValue())
-			currentRound.CTSpend = int64(gs.TeamCounterTerrorists().MoneySpentThisRound())
-			currentRound.TSpend = int64(gs.TeamTerrorists().MoneySpentThisRound())
 
-			currentRound.CTBuyType = parseTeamBuy(currentRound.CTBeginEqVal+currentRound.CTSpend, "CT", currentGame.ParsingOpts.RoundBuyStyle)
-			currentRound.TBuyType = parseTeamBuy(currentRound.TBeginEqVal+currentRound.TSpend, "T", currentGame.ParsingOpts.RoundBuyStyle)
+			currentRound.CTBuyType = parseTeamBuy(currentRound.CTFreezeTimeEndEqVal, "CT", currentGame.ParsingOpts.RoundBuyStyle)
+			currentRound.TBuyType = parseTeamBuy(currentRound.TFreezeTimeEndEqVal, "T", currentGame.ParsingOpts.RoundBuyStyle)
+			// currentRound.CTBuyType = parseTeamBuy(currentRound.CTRoundStartEqVal+currentRound.CTSpend, "CT", currentGame.ParsingOpts.RoundBuyStyle)
+			// currentRound.TBuyType = parseTeamBuy(currentRound.TRoundStartEqVal+currentRound.TSpend, "T", currentGame.ParsingOpts.RoundBuyStyle)
 
-			currentRound.CTStartEqVal = currentRound.CTBeginEqVal + currentRound.CTSpend
-			currentRound.TStartEqVal = currentRound.TBeginEqVal + currentRound.TSpend
-
-			// Parse who won the round
-			// Not great...but a stopgap measure
+			// Parse who won the round, not great...but a stopgap measure
 			tPlayers := gs.TeamTerrorists().Members()
 			aliveT := 0
 			ctPlayers := gs.TeamCounterTerrorists().Members()
@@ -1277,10 +1431,6 @@ func main() {
 				currentRound.TTeam = &tTeam
 				currentRound.CTTeam = &ctTeam
 			}
-
-			// Parse round spend
-			currentRound.TBeginMoney = 800 * 5
-			currentRound.CTBeginMoney = 800 * 5
 		}
 
 		roundInEndTime = 1
@@ -1317,16 +1467,10 @@ func main() {
 			}
 		}
 
-		currentRound.CTBeginEqVal = int64(gs.TeamCounterTerrorists().RoundStartEquipmentValue())
-		currentRound.TBeginEqVal = int64(gs.TeamTerrorists().RoundStartEquipmentValue())
-		currentRound.CTSpend = int64(gs.TeamCounterTerrorists().MoneySpentThisRound())
-		currentRound.TSpend = int64(gs.TeamTerrorists().MoneySpentThisRound())
-
-		currentRound.CTBuyType = parseTeamBuy(currentRound.CTBeginEqVal+currentRound.CTSpend, "CT", currentGame.ParsingOpts.RoundBuyStyle)
-		currentRound.TBuyType = parseTeamBuy(currentRound.TBeginEqVal+currentRound.TSpend, "T", currentGame.ParsingOpts.RoundBuyStyle)
-
-		currentRound.CTStartEqVal = currentRound.CTBeginEqVal + currentRound.CTSpend
-		currentRound.TStartEqVal = currentRound.TBeginEqVal + currentRound.TSpend
+		currentRound.CTBuyType = parseTeamBuy(currentRound.CTFreezeTimeEndEqVal, "CT", currentGame.ParsingOpts.RoundBuyStyle)
+		currentRound.TBuyType = parseTeamBuy(currentRound.TFreezeTimeEndEqVal, "T", currentGame.ParsingOpts.RoundBuyStyle)
+		// currentRound.CTBuyType = parseTeamBuy(currentRound.CTRoundStartEqVal+currentRound.CTSpend, "CT", currentGame.ParsingOpts.RoundBuyStyle)
+		// currentRound.TBuyType = parseTeamBuy(currentRound.TRoundStartEqVal+currentRound.TSpend, "T", currentGame.ParsingOpts.RoundBuyStyle)
 
 	})
 
@@ -1477,6 +1621,9 @@ func main() {
 			currentWeaponFire.PlayerY = float64(playerPos.Y)
 			currentWeaponFire.PlayerZ = float64(playerPos.Z)
 			currentWeaponFire.Weapon = e.Weapon.String()
+			currentWeaponFire.WeaponClass = convertWeaponClass(e.Weapon.Class())
+			currentWeaponFire.AmmoInMagazine = int64(e.Weapon.AmmoInMagazine())
+			currentWeaponFire.AmmoInReserve = int64(e.Weapon.AmmoReserve())
 			currentWeaponFire.PlayerViewX = float64(e.Shooter.ViewDirectionX())
 			currentWeaponFire.PlayerViewY = float64(e.Shooter.ViewDirectionY())
 			currentWeaponFire.PlayerStrafe = e.Shooter.IsWalking()
@@ -1529,53 +1676,55 @@ func main() {
 
 			// Player
 			if e.Player != nil {
-				playerSteamID := int64(e.Player.SteamID64)
-				currentFlash.PlayerSteamID = &playerSteamID
-				currentFlash.PlayerName = &e.Player.Name
-				playerClanName := ""
+				if e.Player.IsAlive() {
+					playerSteamID := int64(e.Player.SteamID64)
+					currentFlash.PlayerSteamID = &playerSteamID
+					currentFlash.PlayerName = &e.Player.Name
+					playerClanName := ""
 
-				if e.Player.TeamState != nil {
-					playerClanName = e.Player.TeamState.ClanName()
-				}
+					if e.Player.TeamState != nil {
+						playerClanName = e.Player.TeamState.ClanName()
+					}
 
-				currentFlash.PlayerTeam = &playerClanName
-				playerSide := "Unknown"
-				switch e.Player.Team {
-				case common.TeamTerrorists:
-					playerSide = "T"
-				case common.TeamCounterTerrorists:
-					playerSide = "CT"
-				case common.TeamSpectators:
-					playerSide = "Spectator"
-				case common.TeamUnassigned:
-					playerSide = "Unassigned"
-				default:
-					playerSide = "Unknown"
-				}
+					currentFlash.PlayerTeam = &playerClanName
+					playerSide := "Unknown"
+					switch e.Player.Team {
+					case common.TeamTerrorists:
+						playerSide = "T"
+					case common.TeamCounterTerrorists:
+						playerSide = "CT"
+					case common.TeamSpectators:
+						playerSide = "Spectator"
+					case common.TeamUnassigned:
+						playerSide = "Unassigned"
+					default:
+						playerSide = "Unknown"
+					}
 
-				currentFlash.PlayerSide = &playerSide
+					currentFlash.PlayerSide = &playerSide
 
-				// Player loc
-				playerPos := e.Player.LastAlivePosition
-				
-				playerX := float64(playerPos.X)
-				playerY := float64(playerPos.Y)
-				playerZ := float64(playerPos.Z)
-				currentFlash.PlayerX = &playerX
-				currentFlash.PlayerY = &playerY
-				currentFlash.PlayerZ = &playerZ
-				playerViewX := float64(e.Player.ViewDirectionX())
-				playerViewY := float64(e.Player.ViewDirectionY())
-				currentFlash.PlayerViewX = &playerViewX
-				currentFlash.PlayerViewY = &playerViewY
+					// Player loc
+					playerPos := e.Player.LastAlivePosition
+					
+					playerX := float64(playerPos.X)
+					playerY := float64(playerPos.Y)
+					playerZ := float64(playerPos.Z)
+					currentFlash.PlayerX = &playerX
+					currentFlash.PlayerY = &playerY
+					currentFlash.PlayerZ = &playerZ
+					playerViewX := float64(e.Player.ViewDirectionX())
+					playerViewY := float64(e.Player.ViewDirectionY())
+					currentFlash.PlayerViewX = &playerViewX
+					currentFlash.PlayerViewY = &playerViewY
 
-				// Calculate flash duration in seconds
-				flashDuration := float64(e.Player.FlashDurationTimeRemaining())/1000000000
-				currentFlash.FlashDuration = &flashDuration
+					// Calculate flash duration in seconds
+					flashDuration := float64(e.Player.FlashDurationTimeRemaining())/1000000000
+					currentFlash.FlashDuration = &flashDuration
 
-				// Add
-				if *currentFlash.PlayerSide != "Spectator" && *currentFlash.PlayerSide != "Unassigned" && *currentFlash.PlayerSide != "Unknown" {
-					currentRound.Flashes = append(currentRound.Flashes, currentFlash)
+					// Add to list
+					if *currentFlash.PlayerSide != "Spectator" && *currentFlash.PlayerSide != "Unassigned" && *currentFlash.PlayerSide != "Unknown" {
+						currentRound.Flashes = append(currentRound.Flashes, currentFlash)
+					}
 				}
 			}
 		}
@@ -1772,12 +1921,122 @@ func main() {
 	p.RegisterEventHandler(func(e events.Kill) {
 		gs := p.GameState()
 
+		if (roundInFreezetime == 0) && (parseKillFrames == true) {
+			currentFrame := GameFrame{}
+			currentFrame.IsKillFrame = true
+
+			// Create empty player lists
+			currentFrame.CT.Players = []PlayerInfo{}
+			currentFrame.T.Players = []PlayerInfo{}
+
+			currentFrame.Tick = int64(gs.IngameTick())
+			currentFrame.Second = determineSecond(currentFrame.Tick, currentRound, currentGame)
+			currentFrame.ClockTime = calculateClocktime(currentFrame.Tick, currentRound, currentGame)
+
+			// Parse T
+			currentFrame.T = TeamFrameInfo{}
+			currentFrame.T.Side = "T"
+			if (gs.TeamTerrorists() != nil) {
+				currentFrame.T.Team = gs.TeamTerrorists().ClanName()
+			}
+			currentFrame.T.CurrentEqVal = int64(gs.TeamTerrorists().CurrentEquipmentValue())
+			tPlayers := gs.TeamTerrorists().Members()
+
+			for _, p := range tPlayers {
+				if p != nil {
+					if playerInList(p, currentFrame.T.Players) == false {
+						currentFrame.T.Players = append(currentFrame.T.Players, parsePlayer(gs, p))
+					}
+				}
+			}
+			
+			currentFrame.T.AlivePlayers = countAlivePlayers(currentFrame.T.Players)
+			currentFrame.T.TotalUtility = countUtility(currentFrame.T.Players)
+			// currentFrame.T.CurrentEqVal = sumPlayerEqVal(currentFrame.T.Players)
+
+			// Parse CT
+			currentFrame.CT = TeamFrameInfo{}
+			currentFrame.CT.Side = "CT"
+			if (gs.TeamCounterTerrorists() != nil) {
+				currentFrame.CT.Team = gs.TeamCounterTerrorists().ClanName()
+			}
+			currentFrame.CT.CurrentEqVal = int64(gs.TeamCounterTerrorists().CurrentEquipmentValue())
+			ctPlayers := gs.TeamCounterTerrorists().Members()
+
+			for _, p := range ctPlayers {
+				if p != nil {
+					if playerInList(p, currentFrame.CT.Players) == false {
+						currentFrame.CT.Players = append(currentFrame.CT.Players, parsePlayer(gs, p))
+					}
+				}
+			}
+			
+			currentFrame.CT.AlivePlayers = countAlivePlayers(currentFrame.CT.Players)
+			currentFrame.CT.TotalUtility = countUtility(currentFrame.CT.Players)
+			// currentFrame.CT.CurrentEqVal = sumPlayerEqVal(currentFrame.CT.Players)
+			
+			// Parse projectiles objects
+			allGrenades := gs.GrenadeProjectiles()
+			currentFrame.Projectiles = []GrenadeInfo{}
+			for _, ele := range allGrenades {
+				currentProjectile := GrenadeInfo{}
+				currentProjectile.ProjectileType = ele.WeaponInstance.String()
+				objPos := ele.Trajectory[len(ele.Trajectory)-1]
+
+				currentProjectile.X = float64(objPos.X)
+				currentProjectile.Y = float64(objPos.Y)
+				currentProjectile.Z = float64(objPos.Z)
+				currentFrame.Projectiles = append(currentFrame.Projectiles, currentProjectile)
+			}
+
+			// Parse infernos
+			allInfernos := gs.Infernos()
+			currentFrame.Fires = []Fire{}
+			for _, ele := range allInfernos {
+				currentFire := Fire{}
+				objPos := ele.Entity.Position()
+				currentFire.UniqueID = ele.UniqueID()
+
+				currentFire.X = float64(objPos.X)
+				currentFire.Y = float64(objPos.Y)
+				currentFire.Z = float64(objPos.Z)
+				currentFrame.Fires = append(currentFrame.Fires, currentFire)
+			}
+
+			// Parse smokes
+			currentFrame.Smokes = []Smoke{}
+			currentFrame.Smokes = smokes
+
+			// Parse bomb
+			bombObj := gs.Bomb()
+			currentBomb := BombInfo{}
+			objPos := bombObj.Position()
+
+			currentBomb.X = float64(objPos.X)
+			currentBomb.Y = float64(objPos.Y)
+			currentBomb.Z = float64(objPos.Z)
+			currentFrame.Bomb = currentBomb
+			if len(currentRound.Bomb) > 0 {
+				for _, b := range currentRound.Bomb {
+					if b.BombAction == "plant" {
+						currentFrame.BombPlanted = true
+						currentFrame.BombSite = *b.BombSite
+					}
+				}	
+			} else {
+				currentFrame.BombPlanted = false
+			}
+
+			currentRound.Frames = append(currentRound.Frames, currentFrame)
+		}
+
 		currentKill := KillAction{}
 		currentKill.Tick = int64(gs.IngameTick())
 		currentKill.Second = determineSecond(currentKill.Tick, currentRound, currentGame)
 		currentKill.ClockTime = calculateClocktime(currentKill.Tick, currentRound, currentGame)
 		if e.Weapon != nil {
 			currentKill.Weapon = e.Weapon.String()
+			currentKill.WeaponClass = convertWeaponClass(e.Weapon.Class())
 		}
 		currentKill.IsWallbang = e.IsWallBang()
 		currentKill.PenetratedObjects = int64(e.PenetratedObjects)
@@ -1971,6 +2230,7 @@ func main() {
 		currentDamage.ClockTime = calculateClocktime(currentDamage.Tick, currentRound, currentGame)
 		if e.Weapon != nil {
 			currentDamage.Weapon = e.Weapon.String()
+			currentDamage.WeaponClass = convertWeaponClass(e.Weapon.Class())
 		}
 		currentDamage.HitGroup = convertHitGroup(e.HitGroup)
 		currentDamage.HpDamage = int64(e.HealthDamage)
@@ -2086,8 +2346,18 @@ func main() {
 	p.RegisterEventHandler(func(e events.FrameDone) {
 		gs := p.GameState()
 
+		if (roundInFreezetime == 0) && (roundInEndTime == 0) {
+			currentRound.CTRoundStartEqVal = int64(gs.TeamCounterTerrorists().RoundStartEquipmentValue())
+			currentRound.TRoundStartEqVal = int64(gs.TeamTerrorists().RoundStartEquipmentValue())
+			currentRound.CTFreezeTimeEndEqVal = int64(gs.TeamCounterTerrorists().FreezeTimeEndEquipmentValue())
+			currentRound.TFreezeTimeEndEqVal = int64(gs.TeamTerrorists().FreezeTimeEndEquipmentValue())
+			currentRound.CTRoundMoneySpend = int64(gs.TeamCounterTerrorists().MoneySpentThisRound())
+			currentRound.TRoundMoneySpend = int64(gs.TeamTerrorists().MoneySpentThisRound())
+		}
+		
 		if (roundInFreezetime == 0) && (currentFrameIdx == 0) && (parseFrames == true) {
 			currentFrame := GameFrame{}
+			currentFrame.IsKillFrame = false
 
 			// Create empty player lists
 			currentFrame.CT.Players = []PlayerInfo{}
@@ -2116,6 +2386,7 @@ func main() {
 			
 			currentFrame.T.AlivePlayers = countAlivePlayers(currentFrame.T.Players)
 			currentFrame.T.TotalUtility = countUtility(currentFrame.T.Players)
+			// currentFrame.T.CurrentEqVal = sumPlayerEqVal(currentFrame.T.Players)
 
 			// Parse CT
 			currentFrame.CT = TeamFrameInfo{}
@@ -2136,31 +2407,49 @@ func main() {
 			
 			currentFrame.CT.AlivePlayers = countAlivePlayers(currentFrame.CT.Players)
 			currentFrame.CT.TotalUtility = countUtility(currentFrame.CT.Players)
+			// currentFrame.CT.CurrentEqVal = sumPlayerEqVal(currentFrame.CT.Players)
 			
-
-			// Parse world (grenade) objects
+			// Parse projectiles objects
 			allGrenades := gs.GrenadeProjectiles()
+			currentFrame.Projectiles = []GrenadeInfo{}
 			for _, ele := range allGrenades {
-				currentWorldObj := WorldObject{}
-				currentWorldObj.ObjType = ele.WeaponInstance.String()
+				currentProjectile := GrenadeInfo{}
+				currentProjectile.ProjectileType = ele.WeaponInstance.String()
 				objPos := ele.Trajectory[len(ele.Trajectory)-1]
 
-				currentWorldObj.X = float64(objPos.X)
-				currentWorldObj.Y = float64(objPos.Y)
-				currentWorldObj.Z = float64(objPos.Z)
-				currentFrame.World = append(currentFrame.World, currentWorldObj)
+				currentProjectile.X = float64(objPos.X)
+				currentProjectile.Y = float64(objPos.Y)
+				currentProjectile.Z = float64(objPos.Z)
+				currentFrame.Projectiles = append(currentFrame.Projectiles, currentProjectile)
 			}
 
+			// Parse infernos
+			allInfernos := gs.Infernos()
+			currentFrame.Fires = []Fire{}
+			for _, ele := range allInfernos {
+				currentFire := Fire{}
+				objPos := ele.Entity.Position()
+				currentFire.UniqueID = ele.UniqueID()
+
+				currentFire.X = float64(objPos.X)
+				currentFire.Y = float64(objPos.Y)
+				currentFire.Z = float64(objPos.Z)
+				currentFrame.Fires = append(currentFrame.Fires, currentFire)
+			}
+
+			// Parse smokes
+			currentFrame.Smokes = []Smoke{}
+			currentFrame.Smokes = smokes
+
 			// Parse bomb
 			bombObj := gs.Bomb()
-			currentWorldObj := WorldObject{}
-			currentWorldObj.ObjType = "bomb"
+			currentBomb := BombInfo{}
 			objPos := bombObj.Position()
 
-			currentWorldObj.X = float64(objPos.X)
-			currentWorldObj.Y = float64(objPos.Y)
-			currentWorldObj.Z = float64(objPos.Z)
-			currentFrame.World = append(currentFrame.World, currentWorldObj)
+			currentBomb.X = float64(objPos.X)
+			currentBomb.Y = float64(objPos.Y)
+			currentBomb.Z = float64(objPos.Z)
+			currentFrame.Bomb = currentBomb
 			if len(currentRound.Bomb) > 0 {
 				for _, b := range currentRound.Bomb {
 					if b.BombAction == "plant" {
